@@ -13,6 +13,12 @@
 
 static int root_warn;
 
+static inline void clear_error(struct thread_data *td)
+{
+	td->error = 0;
+	td->verror[0] = '\0';
+}
+
 /*
  * Leaves f->fd open on success, caller must close
  */
@@ -61,20 +67,28 @@ static int extend_file(struct thread_data *td, struct fio_file *f)
 	if (!new_layout)
 		goto done;
 
-	dprint(FD_FILE, "truncate file %s, size %llu\n", f->file_name,
+	/*
+	 * The size will be -1ULL when fill_device is used, so don't truncate
+	 * or fallocate this file, just write it
+	 */
+	if (!td->o.fill_device) {
+		dprint(FD_FILE, "truncate file %s, size %llu\n", f->file_name,
 							f->real_file_size);
-	if (ftruncate(f->fd, f->real_file_size) == -1) {
-		td_verror(td, errno, "ftruncate");
-		goto err;
-	}
+		if (ftruncate(f->fd, f->real_file_size) == -1) {
+			td_verror(td, errno, "ftruncate");
+			goto err;
+		}
 
 #ifdef FIO_HAVE_FALLOCATE
-	dprint(FD_FILE, "fallocate file %s, size %llu\n", f->file_name,
+		dprint(FD_FILE, "fallocate file %s, size %llu\n", f->file_name,
 							f->real_file_size);
-	r = posix_fallocate(f->fd, 0, f->real_file_size);
-	if (r < 0)
-		log_err("fio: posix_fallocate fails: %s\n", strerror(-r));
+		r = posix_fallocate(f->fd, 0, f->real_file_size);
+		if (r < 0) {
+			log_err("fio: posix_fallocate fails: %s\n",
+					strerror(-r));
+		}
 #endif
+	}
 
 	b = malloc(td->o.max_bs[DDIR_WRITE]);
 	memset(b, 0, td->o.max_bs[DDIR_WRITE]);
@@ -87,13 +101,22 @@ static int extend_file(struct thread_data *td, struct fio_file *f)
 
 		r = write(f->fd, b, bs);
 
-		if (r == (int) bs) {
-			left -= bs;
+		if (r > 0) {
+			left -= r;
 			continue;
 		} else {
-			if (r < 0)
+			if (r < 0) {
+				int __e = errno;
+
+				if (__e == ENOSPC) {
+					if (td->o.fill_device)
+						break;
+					log_info("fio: ENOSPC on laying out "
+						 "file, stopping\n");
+					break;
+				}
 				td_verror(td, errno, "write");
-			else
+			} else
 				td_verror(td, EIO, "write");
 
 			break;
@@ -109,6 +132,13 @@ static int extend_file(struct thread_data *td, struct fio_file *f)
 			goto err;
 		}
 	}
+	if (td->o.fill_device) {
+		fio_file_clear_size_known(f);
+		if (td_io_get_file_size(td, f))
+			goto err;
+		if (f->io_size > f->real_file_size)
+			f->io_size = f->real_file_size;
+	}
 
 	free(b);
 done:
@@ -126,7 +156,7 @@ static int pre_read_file(struct thread_data *td, struct fio_file *f)
 	unsigned int bs;
 	char *b;
 
-	if (!(f->flags & FIO_FILE_OPEN)) {
+	if (!fio_file_open(f)) {
 		if (td->io_ops->open_file(td, f)) {
 			log_err("fio: cannot pre-read, failed to open file\n");
 			return 1;
@@ -154,7 +184,6 @@ static int pre_read_file(struct thread_data *td, struct fio_file *f)
 			left -= bs;
 			continue;
 		} else {
-			printf("r=%d\n", r);
 			td_verror(td, EIO, "pre_read");
 			break;
 		}
@@ -227,7 +256,7 @@ static int get_file_size(struct thread_data *td, struct fio_file *f)
 {
 	int ret = 0;
 
-	if (f->flags & FIO_SIZE_KNOWN)
+	if (fio_file_size_known(f))
 		return 0;
 
 	if (f->filetype == FIO_TYPE_FILE)
@@ -246,7 +275,7 @@ static int get_file_size(struct thread_data *td, struct fio_file *f)
 		return 1;
 	}
 
-	f->flags |= FIO_SIZE_KNOWN;
+	fio_file_set_size_known(f);
 	return 0;
 }
 
@@ -267,8 +296,8 @@ static int __file_invalidate_cache(struct thread_data *td, struct fio_file *f,
 	/*
 	 * FIXME: add blockdev flushing too
 	 */
-	if (f->mmap)
-		ret = madvise(f->mmap, len, MADV_DONTNEED);
+	if (f->mmap_ptr)
+		ret = madvise(f->mmap_ptr, f->mmap_sz, MADV_DONTNEED);
 	else if (f->filetype == FIO_TYPE_FILE) {
 		ret = fadvise(f->fd, off, len, POSIX_FADV_DONTNEED);
 	} else if (f->filetype == FIO_TYPE_BD) {
@@ -298,10 +327,10 @@ static int __file_invalidate_cache(struct thread_data *td, struct fio_file *f,
 
 int file_invalidate_cache(struct thread_data *td, struct fio_file *f)
 {
-	if (!(f->flags & FIO_FILE_OPEN))
+	if (!fio_file_open(f))
 		return 0;
 
-	return __file_invalidate_cache(td, f, -1, -1);
+	return __file_invalidate_cache(td, f, -1ULL, -1ULL);
 }
 
 int generic_close_file(struct thread_data fio_unused *td, struct fio_file *f)
@@ -561,7 +590,7 @@ int setup_files(struct thread_data *td)
 				extend_size += (f->io_size + f->file_offset);
 			} else
 				f->real_file_size = f->io_size + f->file_offset;
-			f->flags |= FIO_FILE_EXTEND;
+			fio_file_set_extend(f);
 		}
 	}
 
@@ -579,20 +608,22 @@ int setup_files(struct thread_data *td)
 					extend_size >> 20);
 
 		for_each_file(td, f, i) {
-			unsigned long long old_len, extend_len;
+			unsigned long long old_len = -1ULL, extend_len = -1ULL;
 
-			if (!(f->flags & FIO_FILE_EXTEND))
+			if (!fio_file_extend(f))
 				continue;
 
 			assert(f->filetype == FIO_TYPE_FILE);
-			f->flags &= ~FIO_FILE_EXTEND;
-			old_len = f->real_file_size;
-			extend_len = f->io_size + f->file_offset - old_len;
+			fio_file_clear_extend(f);
+			if (!td->o.fill_device) {
+				old_len = f->real_file_size;
+				extend_len = f->io_size + f->file_offset - old_len;
+			}
 			f->real_file_size = (f->io_size + f->file_offset);
 			err = extend_file(td, f);
 			if (err)
 				break;
-			
+
 			err = __file_invalidate_cache(td, f, old_len,
 								extend_len);
 			close(f->fd);
@@ -803,7 +834,7 @@ int add_file(struct thread_data *td, const char *fname)
 void get_file(struct fio_file *f)
 {
 	dprint(FD_FILE, "get file %s, ref=%d\n", f->file_name, f->references);
-	assert(f->flags & FIO_FILE_OPEN);
+	assert(fio_file_open(f));
 	f->references++;
 }
 
@@ -813,7 +844,7 @@ int put_file(struct thread_data *td, struct fio_file *f)
 
 	dprint(FD_FILE, "put file %s, ref=%d\n", f->file_name, f->references);
 
-	if (!(f->flags & FIO_FILE_OPEN))
+	if (!fio_file_open(f))
 		return 0;
 
 	assert(f->references);
@@ -830,7 +861,7 @@ int put_file(struct thread_data *td, struct fio_file *f)
 		ret = f_ret;
 
 	td->nr_open_files--;
-	f->flags &= ~FIO_FILE_OPEN;
+	fio_file_clear_open(f);
 	return ret;
 }
 
