@@ -18,8 +18,9 @@
 #include "parse.h"
 #include "smalloc.h"
 #include "filehash.h"
+#include "verify.h"
 
-static char fio_version_string[] = "fio 1.26.4";
+static char fio_version_string[] = "fio 1.27";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -357,6 +358,9 @@ static int fixup_options(struct thread_data *td)
 	if (td->o.verify != VERIFY_NONE)
 		td->o.refill_buffers = 1;
 
+	if (td->o.pre_read)
+		td->o.invalidate_cache = 0;
+
 	return 0;
 }
 
